const http = require('http');
const urlTool = require('url');
const path = require('path');
const fs = require('fs');
const ejs = require('ejs');
const mimes = require('./mimes/mimes');
const zlib = require('zlib');
const etag = require('etag');
const config = require('./config/config');
const open = require('./libs/open');



class H51227 {
    constructor(options){
        Object.assign(config,options);
    }

    run(){
        const server = http.createServer((request,response)=>{
            let url = urlTool.parse(request.url).pathname;


            // let filePath = path.resolve(__dirname,'public'+url);
            let filePath = config.root + url;
            response.setHeader('content-type','text/html;charset=utf-8');
            fs.stat(filePath,(err,stats)=>{
                if (err){
                    response.statusCode = 404;
                    response.end('<h1>Not Find</h1>');
                    return;
                }else {
                    //判断是否为文件夹
                    //是文件夹
                    if (stats.isDirectory()){
                        fs.readdir(filePath,(err,data)=>{
                            if (err){
                                response.statusCode = 500;
                                response.end('500');
                                return;
                            }

                            ejs.renderFile(__dirname + '/views/directory.ejs', {data:data,url:url},(err,data)=>{
                                if (err){
                                    response.end('500');
                                }else {
                                    response.end(data);
                                }
                            })

                        })
                    }else {
                        //    检测是否有缓存
                        //    是否协商缓存
                        if(config.cache.xieCache){
                            let requestEtag = request.headers['if-none-mach'];
                            let requestLastModified = request.headers['if-modified-since'];
                            //    当前etag值
                            let currentFileEtag = etag(stats);
                            let currentFileModiedTime = stats.mtime.toUTCString();
                            if(requestEtag === currentFileEtag || requestLastModified === currentFileModiedTime){
                                response.statusCode = 304;
                                response.end();
                                return;
                            }
                        }


                        //    是文件
                        let suffix = url.split('.').pop();
                        //设置文件类型
                        if(suffix){
                            response.setHeader('content-type', mimes[suffix]);
                        }else {
                            response.setHeader('content-type', 'text/plan; charset=utf-8');
                        }

                        //设置强制缓存
                        //是否开启强制缓存
                        if(config.cache.qiangCache){
                            response.setHeader('cache-control','max-age='+ config.cache.cacheTime);
                        }


                        //设置etag
                        let etagValue = etag(stats);
                        response.setHeader('Etag',etagValue);
                        response.setHeader('last-modified',stats.mtime.toUTCString());



                        //读取文件
                        fs.readFile(filePath,(err,data)=>{
                            if (err){
                                response.end('500');
                            }
                            //设置文件压缩
                            let encoding = request.headers['accept-encoding'];
                            if (encoding === undefined){
                                response.end(data);
                                return;
                            }

                            if (encoding.indexOf('gzip') !== -1){
                                response.setHeader('content-encoding','gzip');
                                zlib.gzip(data,(err,result)=>{
                                    if (err) {
                                        //如果出错
                                        response.statusCode = 500;
                                        response.end('<h1>Interval server Error</h1>')
                                        return;
                                    }
                                    response.end(result);
                                })
                            }else if(encoding.indexOf('deflate') !== -1){
                                response.setHeader('content-encoding','deflate');
                                zlib.deflate(data,(err,result)=>{
                                    if (err) {
                                        //如果出错
                                        response.statusCode = 500;
                                        response.end('<h1>Interval server Error</h1>')
                                        return;
                                    }
                                    response.end(result);
                                })
                            }else if(encoding.indexOf('br') !== -1) {
                                response.setHeader('content-encoding', 'br');
                                zlib.br(data, (err, result) => {
                                    if (err) {
                                        //如果出错
                                        response.statusCode = 500;
                                        response.end('<h1>Interval server Error</h1>')
                                        return;
                                    }
                                    response.end(result);
                                })
                            }

                        })

                    }
                }


            })



        });

        server.listen(config.port,()=>{
            console.log(config.port + '端口服务中');
            open(`http://127.0.0.1:${config.port}`);
        });
    }
}

module.exports = H51227;